package gregapi.lang;

import gregapi.data.MT;
import gregapi.data.OP;
import gregapi.data.TD;
import gregapi.oredict.OreDictMaterial;
import gregapi.oredict.OreDictPrefix;
import gregapi.util.UT;

import java.util.HashMap;
import java.util.Map.Entry;

import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.StatCollector;
import net.minecraftforge.common.config.Configuration;
import net.minecraftforge.common.config.Property;
import cpw.mods.fml.common.registry.LanguageRegistry;

/**
 * @author Gregorius Techneticies
 */
public class LanguageHandler {
    public static Configuration sLangFile;
    
    private static final HashMap<String, String> TEMPMAP = new HashMap<String, String>(), BUFFERMAP = new HashMap<String, String>();
	private static boolean mWritingEnabled = false, mUseFile = false;
	
	public static void save() {
		mWritingEnabled = true;
		sLangFile.save();
	}
	
	public static synchronized void add(String aKey, String aEnglish) {
		if (aKey == null) return;
		aKey = aKey.trim();
		if (sLangFile == null) {
			BUFFERMAP.put(aKey, aEnglish);
		} else {
			if (!BUFFERMAP.isEmpty()) {
				mUseFile = sLangFile.get("EnableLangFile", "UseThisFileAsLanguageFile", false).getBoolean(false);
				for (Entry<String, String> tEntry : BUFFERMAP.entrySet()) {
					Property tProperty = sLangFile.get("LanguageFile", tEntry.getKey(), tEntry.getValue());
					TEMPMAP.put(tEntry.getKey(), mUseFile?tProperty.getString():tEntry.getValue());
					LanguageRegistry.instance().injectLanguage("en_US", TEMPMAP);
					TEMPMAP.clear();
				}
				if (mWritingEnabled) sLangFile.save();
				BUFFERMAP.clear();
			}
			Property tProperty = sLangFile.get("LanguageFile", aKey, aEnglish);
			if (!tProperty.wasRead() && mWritingEnabled) sLangFile.save();
			TEMPMAP.put(aKey, mUseFile?tProperty.getString():aEnglish);
			LanguageRegistry.instance().injectLanguage("en_US", TEMPMAP);
			TEMPMAP.clear();
		}
	}
	
	public static String get(String aKey, String aDefault) {
		add(aKey, aDefault);
		return translate(aKey, aDefault);
	}
	
	public static String translate(String aKey) {
		return translate(aKey, aKey);
	}
	
	public static String translate(String aKey, String aDefault) {
		if (aKey == null) return "";
		aKey = aKey.trim();
		String rTranslation = LanguageRegistry.instance().getStringLocalization(aKey);
		if (UT.Code.stringInvalid(rTranslation)) {
			rTranslation = StatCollector.translateToLocal(aKey);
			if (UT.Code.stringInvalid(rTranslation) || aKey.equals(rTranslation)) {
				if (aKey.endsWith(".name")) {
					rTranslation = StatCollector.translateToLocal(aKey = aKey.substring(0, aKey.length() - 5));
					if (UT.Code.stringInvalid(rTranslation) || aKey.equals(rTranslation)) return aDefault;
				} else {
					rTranslation = StatCollector.translateToLocal(aKey = aKey + ".name");
					if (UT.Code.stringInvalid(rTranslation) || (aKey).equals(rTranslation)) return aDefault;
				}
			}
		}
		return rTranslation;
	}
	
	public static String separate(String aKey, String aSeparator) {
		if (aKey == null) return "";
		String rTranslation = "";
		for (String tString : aKey.split(aSeparator)) {
			rTranslation += get(tString, tString);
		}
		return rTranslation;
	}
	
	public static String getTranslateableItemStackName(ItemStack aStack) {
		if (UT.Stacks.invalid(aStack)) return "null";
		NBTTagCompound tNBT = aStack.getTagCompound();
		if (tNBT != null && tNBT.hasKey("display")) {
			String tName = tNBT.getCompoundTag("display").getString("Name");
			if (UT.Code.stringValid(tName)) {
				return tName;
			}
		}
		return aStack.getUnlocalizedName() + ".name";
	}
	
	public static String getLocalName(OreDictPrefix aPrefix, OreDictMaterial aMaterial) {
		// Certain Materials have slightly different Localisations.
		if (aPrefix == OP.crateGtGem		|| aPrefix == OP.blockGem)		return aPrefix.mMaterialPre + getLocalName(OP.gem		, aMaterial);
		if (aPrefix == OP.crateGtDust		|| aPrefix == OP.blockDust)		return aPrefix.mMaterialPre + getLocalName(OP.dust		, aMaterial);
		if (aPrefix == OP.crateGtIngot		|| aPrefix == OP.blockIngot)	return aPrefix.mMaterialPre + getLocalName(OP.ingot		, aMaterial);
		if (aPrefix == OP.crateGtPlate		|| aPrefix == OP.blockPlate)	return aPrefix.mMaterialPre + getLocalName(OP.plate		, aMaterial);
		if (aPrefix == OP.crateGtPlateGem	|| aPrefix == OP.blockPlateGem)	return aPrefix.mMaterialPre + getLocalName(OP.plateGem	, aMaterial);
		
		if (aMaterial == MT.Empty) {
			if (aPrefix == OP.chemtube)										return "Empty Glass Tube";
		} else
		if (aMaterial == MT.Glass) {
			if (aPrefix.mNameInternal.startsWith("gem"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Crystal";
			if (aPrefix.mNameInternal.startsWith("plate"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Pane";
		} else
		if (aMaterial == MT.InfusedAir || aMaterial == MT.InfusedDull || aMaterial == MT.InfusedEarth || aMaterial == MT.InfusedEntropy || aMaterial == MT.InfusedFire || aMaterial == MT.InfusedOrder || aMaterial == MT.InfusedVis || aMaterial == MT.InfusedWater || aMaterial == MT.InfusedBalance) {
			if (aPrefix.mNameInternal.startsWith("ore"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Infused Stone";
			if (aPrefix.mNameInternal.startsWith("gem"))					return aPrefix.mMaterialPre + "Shard of " + aMaterial.mNameLocal;
			if (aPrefix.mNameInternal.startsWith("crystal"))				return aPrefix.mMaterialPre + "Shard of " + aMaterial.mNameLocal;
			if (aPrefix.mNameInternal.startsWith("plate"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Crystal Plate";
			if (aPrefix.mNameInternal.startsWith("dust"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Crystal Powder";
			if (aPrefix == OP.crushedCentrifuged)							return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Crystals";
			if (aPrefix == OP.crushedPurified)								return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Crystals";
			if (aPrefix == OP.crushed)										return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Crystals";
		} else
		if (aMaterial == MT.Wheat) {
			if (aPrefix.mNameInternal.startsWith("dust"))					return aPrefix.mMaterialPre + "Flour";
		} else
		if (aMaterial == MT.Ice) {
			if (aPrefix.mNameInternal.startsWith("dust"))					return aPrefix.mMaterialPre + "Crushed Ice";
		} else
		if (aMaterial == MT.Wood || aMaterial == MT.WoodSealed || aMaterial == MT.WoodRubber) {
			if (aPrefix.mNameInternal.startsWith("bolt"))					return "Short " + aMaterial.mNameLocal + " Stick";
			if (aPrefix.mNameInternal.startsWith("stick"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Stick";
			if (aPrefix.mNameInternal.startsWith("dust"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Pulp";
			if (aPrefix.mNameInternal.startsWith("nugget"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Chip";
			if (aPrefix.mNameInternal.startsWith("plate"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Plank";
		} else
		if (aMaterial == MT.Plastic || aMaterial == MT.Rubber) {
			if (aPrefix.mNameInternal.startsWith("dust"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Pulp";
			if (aPrefix.mNameInternal.startsWith("plate"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Sheet";
			if (aPrefix.mNameInternal.startsWith("ingot"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Bar";
			if (aPrefix.mNameInternal.startsWith("nugget"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Chip";
			if (aPrefix.mNameInternal.startsWith("foil"))					return "Thin " + aMaterial.mNameLocal + " Sheet";
		} else
		if (aMaterial == MT.FierySteel) {
			if (aPrefix.contains(TD.Prefix.IS_CONTAINER))					return aPrefix.mMaterialPre + "Fiery Blood" + aPrefix.mMaterialPost;
		} else
		if (aMaterial == MT.Steeleaf) {
			if (aPrefix == OP.plantGtBlossom)								return "Steeleaf Leaf";
			if (aPrefix.mNameInternal.startsWith("ingot"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal;
		} else
		if (aMaterial == MT.Bone) {
			if (aPrefix.mNameInternal.startsWith("dust"))					return aPrefix.mMaterialPre + "Bone Meal";
		} else
		if (aMaterial == MT.Fe) {
			if (aPrefix == OP.plantGtBlossom)								return "Ferru Leaf";
		} else
		if (aMaterial == MT.Au) {
			if (aPrefix == OP.plantGtBlossom)								return "Aurelia Leaf";
		} else
		if (aMaterial == MT.Pb) {
			if (aPrefix == OP.plantGtBlossom)								return "Plumbilia Leaf";
		} else
		if (aMaterial == MT.Ag) {
			if (aPrefix == OP.plantGtBlossom)								return "Argentia Leaf";
		} else
		if (aMaterial == MT.Sn) {
			if (aPrefix == OP.plantGtTwig)									return "Tine Twig";
		} else
		if (aMaterial == MT.Cu) {
			if (aPrefix == OP.plantGtFiber)									return "Coppon Fiber";
		} else
		if (aMaterial == MT.Emerald) {
			if (aPrefix == OP.plantGtBerry)									return "Bobs-Yer-Uncle-Berry";
		} else
		if (aMaterial == MT.Milk) {
			if (aPrefix == OP.plantGtWart)									return "Milkwart";
			if (aPrefix.mNameInternal.startsWith("dust"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Powder";
		} else
		if (aMaterial == MT.Blaze || aMaterial == MT.Cocoa || aMaterial == MT.Chocolate || aMaterial == MT.Coffee || aMaterial == MT.Chili || aMaterial == MT.Cheese || aMaterial == MT.Snow) {
			if (aPrefix.mNameInternal.startsWith("dust"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal + " Powder";
		} else
		if (aMaterial == MT.Paper) {
			if (aPrefix.mNameInternal.startsWith("dust"))					return aPrefix.mMaterialPre + "Chad";
			if (aPrefix == OP.plate)										return "Sheet of Paper";
			if (aPrefix == OP.plateDouble)									return "Paperboard";
			if (aPrefix == OP.plateTriple)									return "Carton";
			if (aPrefix == OP.plateQuadruple)								return "Cardboard";
			if (aPrefix == OP.plateQuintuple)								return "Thick Cardboard";
			if (aPrefix == OP.plateDense)									return "Strong Cardboard";
		} else
		if (aMaterial == MT.MeatRaw) {
			if (aPrefix.mNameInternal.startsWith("dust"))					return aPrefix.mMaterialPre + "Mince Meat";
		} else
		if (aMaterial == MT.MeatCooked) {
			if (aPrefix.mNameInternal.startsWith("dust"))					return aPrefix.mMaterialPre + "Cooked Mince Meat";
		} else
		if (aMaterial == MT.Gunpowder || aMaterial == MT.Sugar || aMaterial == MT.Salt || aMaterial == MT.RockSalt) {
			if (aPrefix.mNameInternal.startsWith("dust"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal;
		} else
		if (aMaterial == MT.Ash || aMaterial == MT.DarkAsh || aMaterial == MT.VolcanicAsh || aMaterial == MT.Vermiculite || aMaterial == MT.Bentonite || aMaterial == MT.Kaolinite || aMaterial == MT.Talc || aMaterial == MT.BasalticMineralSand || aMaterial == MT.GraniticMineralSand || aMaterial == MT.GlauconiteSand || aMaterial == MT.CassiteriteSand || aMaterial == MT.GarnetSand || aMaterial == MT.QuartzSand || aMaterial == MT.Pitchblende || aMaterial == MT.FullersEarth || aMaterial == MT.RareEarth || aMaterial == MT.Oilsands) {
			if (aPrefix.mNameInternal.startsWith("ore"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal;
			if (aPrefix.mNameInternal.startsWith("dust"))					return aPrefix.mMaterialPre + aMaterial.mNameLocal;
			if (aPrefix == OP.crushedCentrifuged)							return aPrefix.mMaterialPre + aMaterial.mNameLocal;
			if (aPrefix == OP.crushedPurified)								return aPrefix.mMaterialPre + aMaterial.mNameLocal;
			if (aPrefix == OP.crushed)										return "Ground " + aMaterial.mNameLocal;
		}
		// Use Standard Localisation
		return aPrefix.mMaterialPre + aMaterial.mNameLocal + aPrefix.mMaterialPost;
	}
}