package gregapi.tileentity.cover;

import java.util.List;

import gregapi.render.ITexture;
import net.minecraft.entity.Entity;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.IInventory;
import net.minecraft.item.ItemStack;


/**
 * @author Gregorius Techneticies
 * 
 * Interfaces to use Cover Functionality.
 */
public interface ICover {
	/**
	 * Called when the Cover got successfully placed.
	 * @param aPlayer CAN be null!
	 */
	public void onCoverPlaced(byte aSide, CoverData aData, Entity aPlayer);
	
	/**
	 * @param aData CAN be null!
	 * @param aPlayer CAN be null!
	 * @return true to prevent that the cover gets placed.
	 */
	public boolean interceptCoverPlacement(byte aSide, CoverData aData, Entity aPlayer);
	
	/**
	 * @param aPlayer CAN be null!
	 * @return true to prevent that the cover gets removed.
	 */
	public boolean interceptCoverRemoval(byte aSide, CoverData aData, Entity aPlayer);
	
	/**
	 * Called before the Block does its own Tick. So for example before it checks a Recipe or something.
	 */
	public void onTickPre(byte aSide, CoverData aData, long aTimer, boolean aIsServerSide);
	
	/**
	 * Called after the Block does its own Tick. So for example after it has processed a Recipe or something.
	 */
	public void onTickPost(byte aSide, CoverData aData, long aTimer, boolean aIsServerSide);
	
	/**
	 * Called when the Cover got clicked.
	 * @param aPlayer CAN be null!
	 * @return true to intercept clicking the Side of the normal Block and cause the click animation
	 */
	public boolean onCoverClickedLeft(byte aSide, CoverData aData, Entity aPlayer, byte aSideClicked, float aHitX, float aHitY, float aHitZ);
	
	/**
	 * Called when the Cover got clicked.
	 * @param aPlayer CAN be null!
	 * @return true to intercept clicking the Side of the normal Block without the click animation.
	 */
	public boolean interceptClickLeft(byte aSide, CoverData aData, Entity aPlayer, byte aSideClicked, float aHitX, float aHitY, float aHitZ);
	
	/**
	 * Called when the Cover got clicked.
	 * @param aPlayer CAN be null!
	 * @return true to intercept clicking the Side of the normal Block and cause the click animation
	 */
	public boolean onCoverClickedRight(byte aSide, CoverData aData, Entity aPlayer, byte aSideClicked, float aHitX, float aHitY, float aHitZ);
	
	/**
	 * Called when the Cover got clicked.
	 * @param aPlayer CAN be null!
	 * @return true to intercept clicking the Side of the normal Block without the click animation.
	 */
	public boolean interceptClickRight(byte aSide, CoverData aData, Entity aPlayer, byte aSideClicked, float aHitX, float aHitY, float aHitZ);
	
	/**
	 * Relaying the Tool Clicks to Covers.
	 */
	public long onToolClick(byte aSide, CoverData aData, String aTool, long aRemainingDurability, long aQuality, Entity aPlayer, List<String> aChatReturn, IInventory aPlayerInventory, boolean aSneaking, ItemStack aStack, byte aSideClicked, float aHitX, float aHitY, float aHitZ);
	
	/**
	 * Tooltip List for this Cover Type.
	 */
	public void addToolTips(List aList, ItemStack aStack, boolean aF3_H);
	
	/**
	 * Gets the Item that drops when removing the Cover.
	 */
	public ItemStack getCoverItem(byte aSide, CoverData aData);
	
	/**
	 * @return if you need to save the visual state of this Cover to NBT because the visuals contain crucial information.
	 */
	public boolean needsVisualsSaved(byte aSide, CoverData aData);
	
	public ITexture getCoverTextureSurface(byte aSide, CoverData aData);
	public ITexture getCoverTextureAttachment(byte aSide, CoverData aData, byte aTextureSide);
	public ITexture getCoverTextureHolder(byte aSide, CoverData aData, byte aTextureSide);
	
	public Object getGUIServer(byte aSide, CoverData aData, EntityPlayer aPlayer);
	public Object getGUIClient(byte aSide, CoverData aData, EntityPlayer aPlayer);
}