package gregapi.tileentity.delegate;

import static gregapi.data.CS.FORGE_DIR;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.world.World;
import net.minecraftforge.common.util.ForgeDirection;

/**
 * @author Gregorius Techneticies
 */
public class DelegatorTileEntity<T> {
    /** the TileEntity. This should be an instance of TileEntity. */
	public final T mTileEntity;
	/** the Coordinates the TileEntity has, or should have in case mTileEntity is null. */
	public final int mX, mY, mZ;
	/** the World Object the TileEntity has, or should have in case mTileEntity is null. */
	public final World mWorld;
    /** the Side of the getDelegate responsible for handling. So a TE-Tesseract alike can go a curve. */
	public final byte mSideOfTileEntity;

	public DelegatorTileEntity(DelegatorTileEntity<T> aDelegator) {
		mTileEntity = aDelegator.mTileEntity;
		mWorld = aDelegator.mWorld;
		mX = aDelegator.mX;
		mY = aDelegator.mY;
		mZ = aDelegator.mZ;
		mSideOfTileEntity = aDelegator.mSideOfTileEntity;
	}
	
	public DelegatorTileEntity(T aTileEntity, byte aSideOfTileEntity) {
		mTileEntity = aTileEntity;
		mSideOfTileEntity = aSideOfTileEntity;
		if (aTileEntity instanceof TileEntity) {
			mWorld = ((TileEntity)aTileEntity).getWorldObj();
			mX = ((TileEntity)aTileEntity).xCoord;
			mY = ((TileEntity)aTileEntity).yCoord;
			mZ = ((TileEntity)aTileEntity).zCoord;
		} else {
			throw new IllegalArgumentException("Parameter must be an instance of TileEntity and != null");
		}
	}
	
	public DelegatorTileEntity(T aTileEntity, DelegatorTileEntity aDelegator) {
		mTileEntity = aTileEntity;
		mWorld = aDelegator.mWorld;
		mX = aDelegator.mX;
		mY = aDelegator.mY;
		mZ = aDelegator.mZ;
		mSideOfTileEntity = aDelegator.mSideOfTileEntity;
	}
	
	public DelegatorTileEntity(T aTileEntity, World aWorld, int aX, int aY, int aZ, byte aSideOfTileEntity) {
		mTileEntity = aTileEntity;
		mWorld = aWorld;
		mX = aX;
		mY = aY;
		mZ = aZ;
		mSideOfTileEntity = aSideOfTileEntity;
	}
	
	public ForgeDirection getForgeSideOfTileEntity() {return FORGE_DIR[mSideOfTileEntity];}
}